
// Entry.java

/* Apisitt Rattana, Jan 14, 2001
   Computer Engineering, Prince of Songkla University
   Revised by
     Andrew Davison, dandrew@ratree.psu.ac.th, Feb 2001


 This file contains classes for storing comic title/issue
 information. The classes are:
  Entry:   stores title, issues, imageURL, and source ID info.
           issues is a vector of issue objects;
  Issue:   stores a single num or a range num-num.

*/

import java.io.*;
import java.util.*;
import COM.stevesoft.pat.*; // Regular Expression


public class Entry
// An entry consists of title, issues, imageURL
// and sourceID
{
  String title, imageURL, sourceID;
  Vector issues;     // a vector of nums and num-num ranges


  public Entry(String t, String iss, String iu, String sID)
  {
    title = t.trim();   // get rid of spaces around the title
    imageURL = iu;
	sourceID = sID;
    issues = new Vector();

    // convert the string of nums and num-num ranges into
    // issue objects in the issues vector
    StringTokenizer tokens = new StringTokenizer(iss);
    while (tokens.hasMoreTokens()) {
      Issue i = new Issue( tokens.nextToken() );
      issues.add(i);
    }
  }

  public String getTitle()
  {  return title;  }

  public String getSourceID()
  {  return sourceID; }

  public String toString()   // for debugging
  {
     return " Entry:\n" + 
            "  Title: " + title + "\n" +
            "  Issues: " + issues.toString() + "\n" +
            "  imageURL: " + imageURL + "\n" +
			"  sourceID: " + sourceID + "\n\n";    
  }

  public String numberImage(String issueNo) 
  // replace *'s in imageURL with issueNo
  {
    Regex sStar = new Regex( "\\*{1,}" );   // sequence of stars
    if (sStar.search(imageURL)) {           // if *'s in imageURL
	  int numStars = (sStar.stringMatched()).length();
      while (issueNo.length() < numStars)
	    issueNo = "0" + issueNo;  // left pad with 0's

	  Regex rStar = new Regex("\\*{1,}", issueNo);  // replace *'s by issueNo
      if (sStar.didMatch())
        return rStar.replaceFirst(imageURL);
    }
    return imageURL;
  }  // end of numberImage()



  public boolean matches(String t, int issueNo)
  // Does the title t and issueNo match this entry?
  {
    Regex mTitle = new Regex("(?i)" + t);
    mTitle.search(title);
    if (mTitle.didMatch() && hasIssue(issueNo))
      return true;

    return false;
  }

  boolean hasIssue(int issueNo)
  // Check issueNo against each issue object
  // in the issues vector
  {
    int i = 0;
    int size = issues.size();
    while( i < size) {
      if (((Issue)issues.get(i)).inRange(issueNo))
        return true;
      i++;
    }
    return false;
  }

} // end of Entry class




// **********************************************************

class Issue
// Stores an issue num or a range num-num
{
  boolean isRange;     // distinguishes between the two kinds of info
  int startNum;
  int endNum;

  public Issue(String issue)
  // issue may be a single number or "num-num"
  {
    int dashPosn;
    if ((dashPosn = issue.indexOf('-')) == -1){   // issue is "num"
      startNum = Integer.parseInt(issue);
      endNum = startNum;    // not really necessary
      isRange = false;
    }
    else {     // issues is "num-num"
          startNum = Integer.parseInt( issue.substring(0, dashPosn) );
          endNum = Integer.parseInt( issue.substring(dashPosn+1) );
      isRange = true;
    }
  }


  public boolean inRange(int num)
  // Is num in the range stored here?
  {
    if (!isRange) {    // only a single num stored
      if (num == startNum)
        return true;
     else
        return false;
    }
    else {    // this object stores a range num-num
      if ((num >= startNum) && (num <= endNum))
        return true;
      else
       return false;
    }
  }


  public String toString()   // used for debugging
  {
    if (isRange)
      return startNum + "-" + endNum;
    else
     return startNum + "";
  }

} // end of Issue class
